//
//  AGA10_CpCvHS.c
//  AGA
//
//  Created by Rodrigo Carvajal on 08/Jun/13.
//  Copyright (c) 2013 Rodrigo Carvajal. All rights reserved.
//

#include <stdio.h>
#include <math.h>
#include "AGA8_ZFactor.h"
#include "AGA10_IdealGas.h"
#include "AGA10_CpCvHS.h"

/*
 Information based on AGA Report No. 10, Speed of Sound in Natural Gas and Other Related Hydrocarbon Gases, Catalog #XQ0310, Prepared by Transmission Measurement Committee, 2003 American Gas Association.
 
 */

int AGA_RealGas(const double T, const double P, const double B, const double K, const double C_star_n[],  const double u_n[],
                const int c_n[], const int b_n[], const int k_n[], const double d, const double dBdT, const double d2BdT2,
                const double MF[], const double MW, const double Ru, double Return[])
{
/*
    Inputs:
        T        = Temperature, Kelvin
        P        = Pressure, Pa
        B        = Second Viral Coefficient, m^3/kg-mole
        K        = Mixture Size Parameter, dimensionless
        C_n_star = Array of C_n_star values (see Equation 19)
        u_n      = Array of u_n values from Table 4, dimensionless
        c_n      = Array of c_n values from Table 4, dimensionless
        b_n      = Array of b_n values from Table 4, dimensionless
        k_n      = Array of k_n values from Table 4, dimensionless
        d        = Molar Density, kg-mole/m^3
        dBdT     = Derivative of the Second Viral Coefficient wrt T at constant d
        d2BdT2   = Second derivative of the Second Viral Coefficient wrt T at constant d
        MF[]     = Pointer to array of Mole Fractions of the format:
                    0  Not Used (use zero)
                    1	Methane
                    2	Nitrogen
                    3	Carbon Dioxide
                    4	Ethane
                    5	Propane
                    6	Water
                    7	Hydrogen Sulfide
                    8	Hydrogen
                    9	Carbon Monoxide
                    10	Oxygen
                    11	i-Butane
                    12	n-Butane
                    13	i-Pentane
                    14	n-Pentane
                    15	n-Hexane
                    16	n-Heptane
                    17	n-Octane
                    18	n-Nonane
                    19	n-Decane
                    20	Helium
                    21	Argon
        MW      = Actual Gas molecular weight, kg / kg-mol
        Ru      = Universal Gas Constant, J/mol-K

     Return:
         0 = No Errors
         Other = Errors
         Properties in Return Vector in the following order:
             0 = Isochoric specific heat, Btu/lbm-R
             1 = Isobaric specific heat, Btu/lbm-R
             2 = Speed of Sound, ft/sec
             3 = Isentropic exponent, Dimensionless
             4 = Specific Enthalpy, Btu/lbm
             5 = Specific Entropy, Btu/lbm-R
 
 */
    // Data for Gauss-Kronrod Quadrature Integration
    int GKPoints = 5; // Number of Points for the Gauss-Kronrod Quadrature plus one
    
    long double GKRoots[] = {   0.14887433898163121088,
                                0.43339539412924719080,
                                0.67940956829902440263,
                                0.86506336668898451073,
                                0.97390652851717172008 };
    
    long double GKWeights[] = { 0.29552422471475286217,
                                0.26926671930999634918,
                                0.21908636251598204295,
                                0.14945134915058059038,
                                0.066671344308688137179 };
    
    // Begin Gauss-Kronrod Integration
    int i = 0; // Iteration counter
    
    double HHold  = 0.0;
    double CvHold = 0.0;
    double SHold  = 0.0;
    double D      = 0.0; // Reduced density value, dimensionless
    double dZdT   = 0.0; // Holder for compressibility derivative
    double d2ZdT2 = 0.0; // Holder for compressibility derivative
    double dZdd   = 0.0; // Holder for compressibility derivative
    double ZUsed  = 0.0; // Holder for compressibility
    
    for (i=0; i<GKPoints; i++) {
        // Start on the positive side of the absicssa
        // Set Reduced Density value
        D = K*K*K * d * (1.0 + GKRoots[i]) / 2.0;
        
        // Get Z, dZdT, and d2ZdT2 at the modified reduced density (D) location
        ZUsed = AGA8_ZFactor(T, B, K, C_star_n, u_n, c_n, b_n, k_n, D, dBdT, d2BdT2, &dZdT, &d2ZdT2, &dZdd);
        
        // Get intergation value at this D location
        HHold  += GKWeights[i] * dZdT / (D/(K*K*K));
        CvHold += GKWeights[i] * (T*d2ZdT2 + 2.0*dZdT) / (D/(K*K*K));
        SHold  += GKWeights[i] * ((ZUsed-1.0) + T*dZdT) / (D/(K*K*K));
        
        // Move to the negative side of the absicssa
        // Set Reduced Density value
        D = K*K*K * d * (1.0 - GKRoots[i]) / 2.0;
        
        // Get Z, dZdT, and d2ZdT2 at the modified reduced density (D) location
        ZUsed = AGA8_ZFactor(T, B, K, C_star_n, u_n, c_n, b_n, k_n, D, dBdT, d2BdT2, &dZdT, &d2ZdT2, &dZdd);
        
        // Get intergation value at this D location
        HHold  += GKWeights[i] * dZdT / (D/(K*K*K));
        CvHold += GKWeights[i] * (T*d2ZdT2 + 2.0*dZdT) / (D/(K*K*K));
        SHold  += GKWeights[i] * ((ZUsed-1.0) + T*dZdT) / (D/(K*K*K));
    };
    // Reset compressibility factor
    ZUsed = AGA8_ZFactor(T, B, K, C_star_n, u_n, c_n, b_n, k_n, (K*K*K*d), dBdT, d2BdT2, &dZdT, &d2ZdT2, &dZdd);
    
    // Obtain the ideal gas portions of the solution
    // Set place holders for ideal portions
    double Cpo     = 0.0;
    double Ho      = 0.0;
    double So      = 0.0;
    double Smixing = 0.0;
    
    int Check = AGA10_IdealGas(T, MF, MW, Ru, &Cpo, &Ho, &So, &Smixing);
    if ( Check != 0 ) return -1;  // return on unknown error
    
    // Determine real gas properties
    Return[4]  = (Ho + Ru*T*((ZUsed-1.0) - T*HHold*0.5*d)/MW)*(1.8/4.1868);             // Specific Enthalpy, Eq. 3.7.  Btu/lbm
    Return[5]  = (So + Smixing - Ru*(log(P/(ZUsed*101325.0)) + SHold*0.5*d)/MW)/4.1868; // Specific Entropy, Eq. 3.10. Btu/lbm-R
    CvHold     = Cpo - Ru*(1.0 + T*CvHold*0.5*d)/MW;                                    // Isochoric specific heat, Eq. 3.3. kJ/kg-K
    Cpo        = CvHold + Ru*pow(ZUsed + T*dZdT, 2.0)/((ZUsed + d*dZdd) * MW);          // Isobaric specific heat, Eq. 3.4b. kJ/kg-K
    Ho         = sqrt(1000.0*(Cpo/CvHold)*(Ru*T/MW)*(ZUsed + d*dZdd));                  // Speed of Sound, Eq. 3.1.  m/sec
    Return[3]  = Ho*Ho*MW/(ZUsed*Ru*T*1000.0);                                          // Isentropic exponent, Eq. 3.2. Dimensionaless
    Return[2]  = Ho / 0.3048;                                                           // Speed of Sound, Eq. 3.1.  ft/sec
    Return[0]  = CvHold / 4.1868;                                                       // Isochoric specific heat, Eq. 3.3. Btu/lbm-R
    Return[1]  = Cpo / 4.1868;                                                          // Isobaric specific heat, Eq. 3.4b. Btu/lbm-R
    
//    printf("H=%.6f\tS=%.8f\tCv=%.8f\tCp=%.8f\n\nW=%.6f\tk=%.8f\n\n", HHold, SHold, CvHold, Cp, W, Isen);
    
    return 0; // No errors
}
