//
//  AGA10_IdealGas.c
//  AGA
//
//  Created by Rodrigo Carvajal on 08/Jun/13.
//  Copyright (c) 2013 Rodrigo Carvajal. All rights reserved.
//

#include <stdio.h>
#include <math.h>
#include "AGA10_IdealGas.h"

/*
 
 Information based on AGA Report No. 10, Speed of Sound in Natural Gas and Other Related Hydrocarbon Gases, Catalog #XQ0310, Prepared by Transmission Measurement Committee, 2003 American Gas Association.
 
 */

int AGA10_IdealGas(const double T, const double MF[], const double MW, const double Ru, double *Cpi, double *Hi, double *Si, double *Smixing)
{
/*
    Inputs:
        T        = Temperature, K
        MF[]     = Pointer to array of Mole Fractions of the format:
                    0  Not Used (use zero)
                    1	Methane
                    2	Nitrogen
                    3	Carbon Dioxide
                    4	Ethane
                    5	Propane
                    6	Water
                    7	Hydrogen Sulfide
                    8	Hydrogen
                    9	Carbon Monoxide
                    10	Oxygen
                    11	i-Butane
                    12	n-Butane
                    13	i-Pentane
                    14	n-Pentane
                    15	n-Hexane
                    16	n-Heptane
                    17	n-Octane
                    18	n-Nonane
                    19	n-Decane
                    20	Helium
                    21	Argon
         MW      = Actual Gas molecular weight, kg / kg-mol
         Ru      = Universal Gas Constant, J/mol-K
         Cpi     = Reference to return valve for ideal gas Cp, kJ/kg-K
         Hi      = Reference to return valve for ideal gas H, kJ/kg
         Si      = Reference to return valve for ideal gas S, kJ/kg-K
         Smixing = Reference to return valve for ideal gas Smixing, kJ/kg-K
 
    Return:
        0 = No Errors
        Other = Errors

    */
    
    // Constants from AGA 10 Table 4
                           /* A */  /* B */   /* C */  /* D */    /* E */   /* F */   /* G */  /* H */  /* I */   /* J */   /* K */
    double V[22][11] = {{      0.0,     0.0,      0.0,     0.0,       0.0,      0.0,      0.0,     0.0,      0.0,     0.0,      0.0},
    /*Methane*/         { -29776.4, 7.95454,  43.9417, 1037.09,   1.56373,  813.205, -24.9027, 1019.98, -10.1601, 1070.14, -20.0615},
    /*Nitrogen*/        { -3495.34, 6.95587, 0.272892, 662.738, -0.291318, -680.562,  1.78980, 1740.06,      0.0,   100.0,  4.49823},
    /*Carbon Dioxide*/  {  20.7307, 6.96237,  2.68645, 500.371,  -2.56429, -530.443,  3.91921, 500.198,  2.13290, 2197.22,  5.81381},
    /*Ethane*/          { -37524.4, 7.98139,  24.3668, 752.320,   3.53990,  272.846,  8.44724, 1020.13, -13.2732, 869.510, -22.4010},
    /*Propane*/         { -56072.1, 8.14319,  37.0629, 735.402,   9.38159,  247.190,  13.4556, 1454.78, -11.7342, 984.518, -24.0426},
    /*Water*/           { -13773.1, 7.97183,  6.27078, 2572.63,   2.05010,  1156.72,      0.0,   100.0,      0.0,   100.0, -3.24989},
    /*Hydrogen Sulfide*/{ -10085.4, 7.94680,  -0.0838, 433.801,   2.85539,  843.792,  6.31595, 1481.43, -2.88457, 1102.23, -0.51551},
    /*Hydrogen*/        {  -5565.6, 6.66789,  2.33458, 2584.98,  0.749019,  559.656,      0.0,   100.0,      0.0,   100.0, -7.94821},
    /*Carbon Monoxide*/ { -2753.49, 6.95854,  2.02441, 1541.22,  0.096774,  3674.81,      0.0,   100.0,      0.0,   100.0,  6.23387},
    /*Oxygen*/          { -3497.45, 6.96302,  2.40013, 2522.05,   2.21752,  1154.15,      0.0,   100.0,      0.0,   100.0,  9.19749},
    /*Iso-butane*/      { -72387.0, 17.8143,  58.2062, 1787.39,   40.7621,  808.645,      0.0,   100.0,      0.0,   100.0, -44.1341},
    /*N-Butane*/        { -72674.8, 18.6383,  57.4178, 1792.73,   38.6599,  814.151,      0.0,   100.0,      0.0,   100.0, -46.1938},
    /*Iso-Pentane*/     { -91505.5, 21.3861,  74.3410, 1701.58,   47.0587,  775.899,      0.0,   100.0,      0.0,   100.0, -60.2474},
    /*N-Pentane*/       { -83845.2, 22.5012,  69.5789, 1719.58,   46.2164,  802.174,      0.0,   100.0,      0.0,   100.0, -62.2197},
    /*N-Hexane*/        { -94982.5, 26.6225,  80.3819, 1718.49,   55.6598,  802.069,      0.0,   100.0,      0.0,   100.0, -77.5366},
    /*N-Heptane*/       {-103353.0, 30.4029,  90.6941, 1669.32,   63.2028,  786.001,      0.0,   100.0,      0.0,   100.0, -92.0164},
    /*N-Octane*/        {-109674.0, 34.0847,  100.253, 1611.55,   69.7675,  768.847,      0.0,   100.0,      0.0,   100.0, -106.149},
    /*N-Nonane*/        {-122599.0, 38.5014,  111.446, 1646.48,   80.5015,  781.588,      0.0,   100.0,      0.0,   100.0, -122.444},
    /*N-Decane*/        {-133564.0, 42.7143,  122.173, 1654.85,   90.2255,  785.564,      0.0,   100.0,      0.0,   100.0, -138.006},
    /*Helium*/          {      0.0,   4.968,      0.0,   100.0,       0.0,    100.0,      0.0,   100.0,      0.0,   100.0,   1.8198},
    /*Argon*/           {      0.0,   4.968,      0.0,   100.0,       0.0,    100.0,      0.0,   100.0,      0.0,   100.0,   8.6776}  };
    
    // Obtain the ideal gas properties
    int i = 1; // Iteration counter
    
    // Initialize the solutions to zero
    *Cpi     = 0.0;
    *Hi      = 0.0;
    *Si      = 0.0;
    *Smixing = 0.0;
    
    // Perform iteration over constituents
    for (i=1; i < 22; i++) {
        // Equation 3.5
        *Cpi += MF[i] * ( V[i][1] + V[i][2]*pow((V[i][3]/T)/sinh(V[i][3]/T),2.0) + V[i][4]*pow((V[i][5]/T)/cosh(V[i][5]/T),2.0) +
                          V[i][6]*pow((V[i][7]/T)/sinh(V[i][7]/T),2.0) + V[i][8]*pow((V[i][9]/T)/cosh(V[i][9]/T),2.0) ) ;
        
        // Equation 3.6
        *Hi  += MF[i] * ( V[i][0] + V[i][1]*T + V[i][2]*V[i][3]*cosh(V[i][3]/T)/sinh(V[i][3]/T) - V[i][4]*V[i][5]*tanh(V[i][5]/T) +
                          V[i][6]*V[i][7]*cosh(V[i][7]/T)/sinh(V[i][7]/T) - V[i][8]*V[i][9]*tanh(V[i][9]/T) );
        
        // Equation 3.8
        *Si  += MF[i] * ( V[i][10] + V[i][1]*log(T) + V[i][2]*((V[i][3]/T)*cosh(V[i][3]/T)/sinh(V[i][3]/T) - log(sinh(V[i][3]/T))) -
                          V[i][4]*((V[i][5]/T)*tanh(V[i][5]/T) - log(cosh(V[i][5]/T))) +
                          V[i][6]*((V[i][7]/T)*cosh(V[i][7]/T)/sinh(V[i][7]/T) - log(sinh(V[i][7]/T))) -
                          V[i][8]*((V[i][9]/T)*tanh(V[i][9]/T) - log(cosh(V[i][9]/T))) );
        
        // Equation 3.9
        if (MF[i] != 0.0) { // Cannot take ln(0)
            *Smixing += MF[i] * log(MF[i]);
        };        
    };
    
    // Convert to mass basis and change units to kJ/kg-K
    *Cpi      *= 4.184/MW;
    *Hi       *= 4.184/MW;
    *Si       *= 4.184/MW;
    *Smixing  *= -Ru/MW;
    
    return 0; // No errors
}