//
//  AGA8_Properties.c
//  AGA
//
//  Created by Rodrigo Carvajal on 01/Mar/13.
//  Copyright (c) 2013 Rodrigo Carvajal. All rights reserved.
//

#include <stdio.h>
#include <math.h>
#include "AGA8_Properties.h"
#include "AGA8_ZFactor.h"
#include "AGA10_IdealGas.h"
#include "AGA10_CpCvHS.h"
#include "LGE_Viscosity.h"

/* 
 Information based on Compressibility Factors of Natural Gas and Other Related Hydrocarbon Gases by K.E. Starling and J.L. Savidge. Second Edition, November 1992. 2nd Printing July 1994.
*/

double AGA8_Properties(double P, double T, double MF[], int Property)
{
    /*
     Inputs:
        P    = Pressure, psia
        T    = Temperature, ˚F
        MF[] = Pointer to array of Mole Fractions of the format:
                     0  Not Used (use zero)
                     1	Methane
                     2	Nitrogen
                     3	Carbon Dioxide
                     4	Ethane
                     5	Propane
                     6	Water
                     7	Hydrogen Sulfide
                     8	Hydrogen
                     9	Carbon Monoxide
                     10	Oxygen
                     11	i-Butane
                     12	n-Butane
                     13	i-Pentane
                     14	n-Pentane
                     15	n-Hexane
                     16	n-Heptane
                     17	n-Octane
                     18	n-Nonane
                     19	n-Decane
                     20	Helium
                     21	Argon
     
        Property = Indication of what to return,
                     1 = Molecular Weight
                     2 = Second Viral Coefficient, m^3/kg-mole
                     3 = Density, lbm/ft^3
                     4 = Compressibility Factor
                     5 = Isochoric specific heat, Btu/lbm-R
                     6 = Isobaric specific heat, Btu/lbm-R
                     7 = Speed of Sound, ft/sec
                     8 = Isentropic exponent, Dimensionless
                     9 = Specific Enthalpy, Btu/lbm
                    10 = Specific Entropy, Btu/lbm-R
                    11 = Dynamic Viscosity, cP
                    12 = dBdT, Derivative of the Second Viral Coefficient wrt T at constant d
                    13 = d2BdT2, Second derivative of the Second Viral Coefficient wrt T at constant d
     
        Errors:
            -1000.0 =   Too many iterations
            -2000.0 =   Unknown Property
            -3000.0 =   Real Gas Error
     
     */
    
    // Individual Component Molecular Weights. Taken from Table 5.
    double ComponentMolecularWeight[22] = {   0.0,
                                             16.0430, // Methane
                                             28.0135, // Nitrogen
                                             44.0100, // Carbon Dioxide
                                             30.0700, // Ethane
                                             44.0970, // Propane
                                             18.0153, // Water
                                             34.0820, // Hydrogen Sulfide
                                              2.0159, // Hydrogen
                                             28.0100, // Carbon Monoxide
                                             31.9988, // Oxygen
                                             58.1230, // i-Butane
                                             58.1230, // n-Butane
                                             72.1500, // i-Pentane
                                             72.1500, // n-Pentane
                                             86.1770, // n-Hexane
                                            100.2040, // n-Heptane
                                            114.2310, // n-Octane
                                            128.2580, // n-Nonane
                                            142.2850, // n-Decane
                                              4.0026, // Helium
                                             39.9480 }; // Argon
    // Set number of constituents
    int NumberOfConstituents = 21;
    
    
    // Determine mixture molecular weight and summation of mole fractions.
    int    Counter                = 0;
    double Summer                 = 0.0; // Summation of all mole fractions (to equal 100%)
    double MixtureMolecularWeight = 0.0; // Summation for mixture molecular weight
    double MF_used[NumberOfConstituents];
    
    for (Counter=1; Counter<=NumberOfConstituents; Counter++) Summer += MF[Counter];  // Sum total number of constituents
    
    for (Counter=1; Counter<=NumberOfConstituents; Counter++) {
        // First normalize the mass fractions based on total sum of constituents obtained above
        MF_used[Counter] = MF[Counter]/Summer;
        
        // Determine the Molecular Weight based on the normalized mass fractions.
        MixtureMolecularWeight += MF_used[Counter]*ComponentMolecularWeight[Counter];
    };
    
    if ( Property == 1 ) return MixtureMolecularWeight;  // Return Mixture Molecular if Requested
    
    // If not the Mixture Molecular Weight, continue...
    
    // Proceed to determine the Second Virial Coefficient
    
    // Setup the Table 4 parameters    
    double a_n[59] = { 0.0, 0.153832600, 1.341953000,-2.998583000,-0.048312280,  0.375796500, -1.589575000,-0.053588470, 0.886594630, -0.710237040,
                           -1.471722000, 1.321850350,-0.786659250, 2.291290e-09, 0.157672400, -0.436386400,-0.044081590,-0.003433888,  0.032059050,
                            0.024873550, 0.073322790,-0.001600573, 0.642470600, -0.416260100, -0.066899570, 0.279179500,-0.696605100, -0.002860589,
                           -0.008098836, 3.150547000, 0.007224479,-0.705752900,  0.534979200, -0.079314910,-1.418465000,-5.999050e-17, 0.105840200,
                            0.034317290,-0.007022847, 0.024955870, 0.042968180,  0.746545300, -0.291961300, 7.294616000,-9.936757000, -0.005399808,
                           -0.243256700, 0.049870160, 0.003733797, 1.874951000,  0.002168144, -0.658716400, 0.000205518, 0.009776195, -0.020487080,
                            0.015573220, 0.006862415,-0.001226752, 0.002850908 };

    double u_n[59] = { 0.0, 0.0,  0.5,  1.0,  3.5, -0.5,  4.5, 0.5,  7.5, 9.5, 6.0, 12.0, 12.5,  -6.0,  2.0, 3.0,  2.0, 2.0, 11.0, -0.5, 0.5,  0.0,  4.0,
                            6.0, 21.0, 23.0, 22.0, -1.0, -0.5, 7.0, -1.0, 6.0, 4.0,  1.0,  9.0, -13.0, 21.0, 8.0, -0.5, 0.0,  2.0,  7.0, 9.0, 22.0, 23.0,
                            1.0,  9.0,  3.0,  8.0, 23.0,  1.5, 5.0, -0.5, 4.0, 7.0,  3.0,  0.0,   1.0,  0.0  };

    int b_n[59] = { 0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,2,2,2,2,2,2,2,2,2,3,3,3,3,3,3,3,3,3,3,4,4,4,4,4,4,4,5,5,5,5,5,6,6,7,7,8,8,8,9,9 };

    int c_n[59] = { 0,0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,1,0,0,1,1,1,1,1,1,1,0,1,1,1,1,1,1,1,1,1,0,0,1,1,1,1,1,0,1,1,1,1,0,1,0,1,1,1,1,1,1 };
    
    int k_n[59] = { 0,0,0,0,0,0,0,0,0,0,0,0,0,3,2,2,2,4,4,0,0,2,2,2,4,4,4,4,0,1,1,2,2,3,3,4,4,4,0,0,2,2,2,4,4,0,2,2,4,4,0,2,0,2,1,2,2,2,2 };
    
    int g_n[59] = { 0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,1,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,1,0,1,0,0 };
    
    int q_n[59] = { 0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,1,0,0,0,0,1,0,0,0,0,1,0,1,0,0,1,0,0,0,0,0,1 };
    
    int f_n[59] = { 0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,1,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 };

    int s_n[59] = { 0,0,0,0,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 };

    int w_n[59] = { 0,0,0,0,0,0,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0 };
    
    // Setup the Table 5 values.    
    double E[22] = { 0.0, 151.318300,  99.737780, 241.960600, 244.166700, 298.118300, 514.015600, 296.355000,  26.957940, 105.534800, 122.766700,
                          324.068900, 337.638900, 365.599900, 370.682300, 402.636293, 427.722630, 450.325022, 470.840891, 489.558373,   2.610111,
                          119.629900 };
    
    double K[22] = { 0.0, 0.4619255, 0.4479153, 0.4557489, 0.5279209, 0.5837490, 0.3825868, 0.4618263, 0.3514916, 0.4533894, 0.4186954,
                          0.6406937, 0.6341423, 0.6738577, 0.6798307, 0.7175118, 0.7525189, 0.7849550, 0.8152731, 0.8437826, 0.3589888,
                          0.4216551 };

    double G[22] = { 0.0, 0.000000, 0.027815, 0.189065, 0.079300, 0.141239, 0.332500, 0.088500, 0.034369, 0.038953, 0.021000, 0.256692, 0.281835,
                          0.332267, 0.366911, 0.289731, 0.337542, 0.383381, 0.427354, 0.469659, 0.000000, 0.000000 };
    
    double Q[22] = { 0.0, 0.0, 0.0, 0.690000, 0.0, 0.0, 1.067750, 0.633276, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
    
    double F[22] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
    
    double S[22] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.582200, 0.390000, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };

    double W[22] = { 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 1.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0 };
    
    // Setup the Table 6 values.
    double E_star_ij[22][22], U_ij[22][22], K_ij[22][22], G_star_ij[22][22], G_ij[22][22], E_ij[22][22];

    int i, j, n;  // Iteration counters
    
    for (i=1; i<=NumberOfConstituents; i++) {
        for (j=1; j<=NumberOfConstituents; j++) {
            E_star_ij[i][j] = U_ij[i][j] = K_ij[i][j] = G_star_ij[i][j] = E_ij[i][j] = G_ij[i][j] = 1.0;
        };
    };
    
    // Fill in E*_ij values
    E_star_ij[1][2]  = 0.971640;
    E_star_ij[1][3]  = 0.960644;
    E_star_ij[1][5]  = 0.994635;
    E_star_ij[1][6]  = 0.708218;
    E_star_ij[1][7]  = 0.931484;
    E_star_ij[1][8]  = 1.170520;
    E_star_ij[1][9]  = 0.990126;
    E_star_ij[1][11] = 1.019530;
    E_star_ij[1][12] = 0.989844;
    E_star_ij[1][13] = 1.002350;
    E_star_ij[1][14] = 0.999268;
    E_star_ij[1][15] = 1.107274;
    E_star_ij[1][16] = 0.880880;
    E_star_ij[1][17] = 0.880973;
    E_star_ij[1][18] = 0.881067;
    E_star_ij[1][19] = 0.881161;
    
    E_star_ij[2][3]  = 1.022740;
    E_star_ij[2][4]  = 0.970120;
    E_star_ij[2][5]  = 0.945939;
    E_star_ij[2][6]  = 0.746954;
    E_star_ij[2][7]  = 0.902271;
    E_star_ij[2][8]  = 1.086320;
    E_star_ij[2][9]  = 1.005710;
    E_star_ij[2][10] = 1.021000;
    E_star_ij[2][11] = 0.946914;
    E_star_ij[2][12] = 0.973384;
    E_star_ij[2][13] = 0.959340;
    E_star_ij[2][14] = 0.945520;

    E_star_ij[3][4]  = 0.925053;
    E_star_ij[3][5]  = 0.960237;
    E_star_ij[3][6]  = 0.849408;
    E_star_ij[3][7]  = 0.955052;
    E_star_ij[3][8]  = 1.281790;
    E_star_ij[3][9]  = 1.500000;
    E_star_ij[3][11] = 0.906849;
    E_star_ij[3][12] = 0.897362;
    E_star_ij[3][13] = 0.726255;
    E_star_ij[3][14] = 0.859764;
    E_star_ij[3][15] = 0.855134;
    E_star_ij[3][16] = 0.831229;
    E_star_ij[3][17] = 0.808310;
    E_star_ij[3][18] = 0.786323;
    E_star_ij[3][19] = 0.765171;

    E_star_ij[4][5]  = 1.022560;
    E_star_ij[4][6]  = 0.693168;
    E_star_ij[4][7]  = 0.946871;
    E_star_ij[4][8]  = 1.164460;
    E_star_ij[4][12] = 1.013060;
    E_star_ij[4][14] = 1.005320;

    E_star_ij[5][8]  = 1.034787;
    E_star_ij[5][12] = 1.004900;

    E_star_ij[7][15] = 1.008692;
    E_star_ij[7][16] = 1.010126;
    E_star_ij[7][17] = 1.011501;
    E_star_ij[7][18] = 1.012821;
    E_star_ij[7][19] = 1.014089;
    
    E_star_ij[8][9]  = 1.100000;
    E_star_ij[8][11] = 1.300000;
    E_star_ij[8][12] = 1.300000;
    
    // Fill in U_ij values
    U_ij[1][2]  = 0.886106;
    U_ij[1][3]  = 0.963827;
    U_ij[1][5]  = 0.990877;
    U_ij[1][7]  = 0.736833;
    U_ij[1][8]  = 1.156390;
    U_ij[1][12] = 0.992291;
    U_ij[1][14] = 1.003670;
    U_ij[1][15] = 1.302576;
    U_ij[1][16] = 1.191904;
    U_ij[1][17] = 1.205769;
    U_ij[1][18] = 1.219634;
    U_ij[1][19] = 1.233498;

    U_ij[2][3]  = 0.835058;
    U_ij[2][4]  = 0.816431;
    U_ij[2][5]  = 0.915502;
    U_ij[2][7]  = 0.993476;
    U_ij[2][8]  = 0.408838;
    U_ij[2][12] = 0.993556;

    U_ij[3][4]  = 0.969870;
    U_ij[3][7]  = 1.045290;
    U_ij[3][9]  = 0.900000;
    U_ij[3][15] = 1.066638;
    U_ij[3][16] = 1.077634;
    U_ij[3][17] = 1.088178;
    U_ij[3][18] = 1.098291;
    U_ij[3][19] = 1.108021;

    U_ij[4][5]  = 1.065173;
    U_ij[4][7]  = 0.971926;
    U_ij[4][8]  = 1.616660;
    U_ij[4][11] = 1.250000;
    U_ij[4][12] = 1.250000;
    U_ij[4][13] = 1.250000;
    U_ij[4][14] = 1.250000;

    U_ij[7][15] = 1.028973;
    U_ij[7][16] = 1.033754;
    U_ij[7][17] = 1.038338;
    U_ij[7][18] = 1.042735;
    U_ij[7][19] = 1.046966;
  
    // Fill in K_ij values    
    K_ij[1][2]  = 1.003630;
    K_ij[1][3]  = 0.995933;
    K_ij[1][5]  = 1.007619;
    K_ij[1][7]  = 1.000080;
    K_ij[1][8]  = 1.023260;
    K_ij[1][12] = 0.997596;
    K_ij[1][14] = 1.002529;
    K_ij[1][15] = 0.982962;
    K_ij[1][16] = 0.983565;
    K_ij[1][17] = 0.982707;
    K_ij[1][18] = 0.981849;
    K_ij[1][19] = 0.980991;
    
    K_ij[2][3]  = 0.982361;
    K_ij[2][4]  = 1.007960;
    K_ij[2][7]  = 0.942596;
    K_ij[2][8]  = 1.032270;
    
    K_ij[3][4]  = 1.008510;
    K_ij[3][7]  = 1.007790;
    K_ij[3][15] = 0.910183;
    K_ij[3][16] = 0.895362;
    K_ij[3][17] = 0.881152;
    K_ij[3][18] = 0.867520;
    K_ij[3][19] = 0.854406;
    
    K_ij[4][5]  = 0.986893;
    K_ij[4][7]  = 0.999969;
    K_ij[4][8]  = 1.020340;

    K_ij[7][15] = 0.968130;
    K_ij[7][16] = 0.962870;
    K_ij[7][17] = 0.957828;
    K_ij[7][18] = 0.952441;
    K_ij[7][19] = 0.948338;
    
    // Fill in G*_ij values    
    G_star_ij[1][3]  = 0.807653;
    G_star_ij[1][8]  = 1.957310;
    
    G_star_ij[2][3]  = 0.982746;
    
    G_star_ij[3][4]  = 0.370296;
    G_star_ij[3][6]  = 1.673090;
    
    // Initialize the E_ij and G_ij matrices
    for (i=1; i<=NumberOfConstituents; i++) {
        for (j=1; j<=NumberOfConstituents; j++) {
            E_ij[i][j] = E_star_ij[i][j] * pow(E[i]*E[j], 0.5);  // Equation 17
            G_ij[i][j] = G_star_ij[i][j] * (G[i]+G[j])  * 0.5;   // Equation 18
        };
    };
    
    // Convert Temperature to Kelvin
    double T_K = (T-32.0)/1.8 + 273.15;
    
    // Begin looping for determination of the Second Viral Coefficient, B
    
    // Initialize intermediate parameters    
    double B_nij;
    double Inner, Outer;
    double B = 0.0;  // Second Viral Coefficient, Equation 15, m^3/kg-mole
    double dBdT = 0.0;  // Derivative of Second Viral Coefficient wrt T at constant d (for AGA 10 Property Evaluations)
    double d2BdT2 = 0.0;  // Second derivative of the Second Viral Coefficient wrt T at constant d (for AGA 10 Property Evaluations)
    double Factor = 1.0;
    
    for (n=1; n<19; n++) {
        Outer = 0.0;  // Reset outer counter
        
        for (i=1; i<=NumberOfConstituents; i++) {
            Inner = 0.0;  // Reset inner counter
            
            for (j=i; j<=NumberOfConstituents; j++) {
                
                if ( i != j ) Factor = 2.0;
                else          Factor = 1.0;
                
                B_nij = pow(G_ij[i][j]+1.0-g_n[n],g_n[n]) * pow(Q[i]*Q[j]+1.0-q_n[n],q_n[n]) * pow(pow(F[i],0.5)*pow(F[j],0.5)+1.0-f_n[n],f_n[n]) *
                        pow( S[i]*S[j]+1.0-s_n[n],s_n[n]) * pow(W[i]*W[j]+1.0-w_n[n],w_n[n]);  // Equation 16

                Inner += Factor * MF_used[i] * MF_used[j] * pow(E_ij[i][j], u_n[n]) * pow(K[i]*K[j],1.5) * B_nij;  // Portion of Equation 15
            };
            
            Outer += Inner;
        };
        
        B      +=  a_n[n] * pow(T_K,-u_n[n]) * Outer;  // Equation 16
        dBdT   += -u_n[n] * a_n[n] * pow(T_K,-u_n[n]-1.0) * Outer;  // AGA 10 Equation 3.14
        d2BdT2 +=  u_n[n] * (u_n[n]+1.0) * a_n[n] * pow(T_K,-u_n[n]-2.0) * Outer;  // AGA 10 Equation 3.16
    };
    
    if ( Property == 2  ) return B;       // Return Second Viral Coefficient if requested
    if ( Property == 12 ) return dBdT;    // Return derivative of Second Viral Coefficient wrt T at constant d if requested
    if ( Property == 13 ) return d2BdT2;  // Return second derivative of the Second Viral Coefficient wrt T at constant d if requested
    
    // If, not continue to determine C*n    
    double C_star_n[59];
    double Q_hold, F_hold;
    double U_hold=0.0, G_hold=0.0, K_hold=0.0;
    
    // Only value for which F is non-zero is Hydrogen (8)
    F_hold = pow(MF_used[8],2.0)*F[8];  // Equation 23
    
    // Only values for which Q is non-zero are Carbon Dioxide (3), Water (6), and Hydrogen Sulfide (7)
    Q_hold = MF_used[3]*Q[3] + MF_used[6]*Q[6] + MF_used[7]*Q[7]; // Equation 22
    
    // Determine G from Equation 21
    // For values i > 3 and j > 8, G*_ij is 1 and therefore, (G*_ij-1)=0 and this contributes nothing to the summation
    for (i=1; i<4; i++) {
        for (j=i+1; j<9; j++) {
            G_hold += MF_used[i] * MF_used[j] * (G_star_ij[i][j]-1.0) * (G[i]+G[j]);
        };
    };
    
    for (i=1; i<=NumberOfConstituents; i++) G_hold += MF_used[i] * G[i];
    
    // Determine U from Equation 20    
    for (i=1; i<=NumberOfConstituents; i++) U_hold += MF_used[i] * pow(E[i],2.5);
    
    U_hold = pow(U_hold,2.0);
    
    // For values i > 7 and j > 19, U_ij is 1 and therefore, (U_ij^5-1)=0 and this contributes nothing to the summation
    for (i=1; i<8; i++) {
        for (j=i+1; j<20; j++) {
            U_hold += 2.0 * MF_used[i] * MF_used[j] * (pow(U_ij[i][j],5.0)-1.0) * pow(E[i]*E[j],2.5);
        };
    };
    
    U_hold = pow(U_hold,0.20); // Equation 20 is U^5.
    
    // Determine C*_n values from Equation 19
    for (i=13; i<59; i++) {
        C_star_n[i] = a_n[i] * pow(G_hold+1.0-g_n[i],g_n[i]) * pow(pow(Q_hold,2.0)+1.0-q_n[i],q_n[i]) * pow(F_hold+1.0-f_n[i],f_n[i]) * pow(U_hold,u_n[i]);
    };
    
    // Determine K value, Equation 14
    for (i=1; i<=NumberOfConstituents; i++) K_hold += MF_used[i] * pow(K[i],2.5);

    K_hold = pow(K_hold,2.0);
    
    // For values i > 7 and j > 19, K_ij is 1 and therefore, (K_ij^5-1)=0 and this contributes nothing to the summation
    for (i=1; i<8; i++) {
        for (j=i+1; j<20; j++) {
            K_hold += 2.0 * MF_used[i] * MF_used[j] * (pow(K_ij[i][j],5.0)-1.0) * pow(K[i]*K[j],2.5);
        };
    };
    
    K_hold = pow(K_hold,0.2);  // Equation 14 is K^5
    
    //Begin iteration for determination of Compressibility Factor (Z)
    double Z_Factor = 0.95;     // Initial guess
    double R_u      = 8.314510; // Univeral Gas Constant, J/mol-K
    double dZdT     = 0.0;      // Holder for the Derivative of Compressibility wrt T at constant d
    double d2ZdT2   = 0.0;      // Holder for the Second derivative of Compressibility wrt T at constant d
    double dZdd     = 0.0;      // Holder for the Derivative of Compressibility wrt molar density at constant T
    
    double P_Pa = P * (9.80665*0.45359237*144.0)/(pow(0.3048,2.0)); // Convert input psia to Pa
    double D    = P_Pa * pow(K_hold,3.0) / (Z_Factor*R_u*T_K*1000.0);  // Initial guess
    
    double Err_Func = 0.0;     // Error in pressure solution
    double Delta_D  = 1.0e+17; // Incremental difference in D between iterations
    double Z_old    = 0.0;     // Previous iterations compressibility factor
    double tol      = 1.0e-08; // AGA 8 tolerance is 1.0e-06 (see Item 4 in Appendix A.2, page 38), use a little more stringent here
    double D_step   = 0.01;    // Utilize a 1% step when determining the derivative

    int    maxIterations = 100;
    
    for (i=0; i<maxIterations; i++) {
        // Determine compresibility factor at current reduced density
        Z_Factor = AGA8_ZFactor(T_K, B, K_hold, C_star_n, u_n, c_n, b_n, k_n, D, dBdT, d2BdT2, &dZdT, &d2ZdT2, &dZdd);
        
        // Determine error in the pressure function
        Err_Func = P_Pa - Z_Factor*D*R_u*T_K*1000.0/pow(K_hold,3.0); // Get error in pressure calculation
        
        // When both the pressure andreduced density are within tolerance, exit.
        if ( (fabs(Err_Func/P_Pa)<tol) && (fabs(Delta_D/D)<tol) ) {
            tol = 1.0;
            break;
        };
        
        // Store previous iteration compressibility factor
        Z_old = Z_Factor;
        
        // Determine slope factor
        Z_Factor = AGA8_ZFactor(T_K, B, K_hold, C_star_n, u_n, c_n, b_n, k_n, D*(1.0-D_step),dBdT, d2BdT2, &dZdT, &d2ZdT2, &dZdd);
        
        Delta_D = -Err_Func * pow(K_hold, 3.0) * D_step / (1000.0 * R_u * T_K * (Z_old - (1.0-D_step)*Z_Factor));
        
        D -= Delta_D;
    };
    
    if ( tol != 1.0 ) return -1000.0;
    
    double d = D / (K_hold*K_hold*K_hold);  // molar density, kg-mol/m^3
    
    if ( Property == 3 )  return MixtureMolecularWeight*d*pow(0.3048,3.0)/0.45359237; // Density, lbm/ft^3

    if ( Property == 4 )  return Z_Factor;
    
    if ( Property == 11 ) return LGE_Viscosity(T_K, MixtureMolecularWeight*d, MixtureMolecularWeight);
    
    // For additional properties, continue on to real gas function
    // Define the vector for the return properties
    double GetProperties[6] = {0.0, 0.0, 0.0, 0.0, 0.0, 0.0};
    
    int Check = AGA_RealGas(T_K, P_Pa, B, K_hold, C_star_n, u_n, c_n, b_n, k_n, d, dBdT, d2BdT2, MF_used, MixtureMolecularWeight, R_u, GetProperties);
    if ( Check != 0 ) return -3000.0;  // return on unknown error
    
    if ( Property == 5 )  return GetProperties[0];
    if ( Property == 6 )  return GetProperties[1];
    if ( Property == 7 )  return GetProperties[2];
    if ( Property == 8 )  return GetProperties[3];
    if ( Property == 9 )  return GetProperties[4];
    if ( Property == 10 ) return GetProperties[5];

    return -2000.0; // Unknown property
}
