//
//  AGA8_ZFactor.c
//  AGA
//
//  Created by Rodrigo Carvajal on 02/Mar/13.
//  Copyright (c) 2013 Rodrigo Carvajal. All rights reserved.
//

#include <stdio.h>
#include <math.h>
#include "AGA8_ZFactor.h"

/*
 Information based on Compressibility Factors of Natural Gas and Other Related Hydrocarbon Gases by K.E. Starling and J.L. Savidge. Second Edition, November 1992. 2nd Printing July 1994.
 
 Information based on AGA Report No. 10, Speed of Sound in Natural Gas and Other Related Hydrocarbon Gases, Catalog #XQ0310, Prepared by Transmission Measurement Committee, 2003 American Gas Association.
 */

double  AGA8_ZFactor(const double T, const double B, const double K, const double C_star_n[],  const double u_n[],
                     const int c_n[], const int b_n[], const int k_n[], const double D, const double dBdT, const double d2BdT2, double *dZdT, double *d2ZdT2, double *dZdd)
{
    /*
     Inputs:
        T        = Temperature, Kelvin
        B        = Second Viral Coefficient, m^3/kg-mole
        K        = Mixture Size Parameter, dimensionless
        C_n_star = Array of C_n_star values (see Equation 19)
        u_n      = Array of u_n values from Table 4, dimensionless
        c_n      = Array of c_n values from Table 4, dimensionless
        b_n      = Array of b_n values from Table 4, dimensionless
        k_n      = Array of k_n values from Table 4, dimensionless
        D        = Reduced density, dimensionless
        dBdT     = Derivative of the Second Viral Coefficient wrt T at constant d
        d2BdT2   = Second derivative of the Second Viral Coefficient wrt T at constant d
        dZdT     = Reference to the Derivative of Compressibility wrt T at constant d
        d2ZdT2   = Reference to the Second derivative of Compressibility wrt T at constant d
     
     Results
        Z Factor = See Equation 12
        dZdT     = In Reference to the Derivative of Compressibility wrt T at constant d
        d2ZdT2   = In Reference to the Second derivative of Compressibility wrt T at constant d
        dZdd     = In Reference to the Derivative of Compressibility wrt molar density at constant T
     
     Errors:
        None
     */

    // Initialize variables
    double Z_Factor = 1.0 + D*B/pow(K,3.0);  // Final solution for Z-Factor
    *dZdT     = D*dBdT/pow(K,3.0);  // Final solution for dZ/dT  AGA 10 Equation 3.13
    *d2ZdT2   = D*d2BdT2/pow(K,3.0);  // Final solution for d2Z/dT2 AGA 10 Equation 3.15
    *dZdd     = B/pow(K, 3.0);          // Final solution for dZdd AGA 10 Equation 3.17
    double Hold     = 0.0;
    
    int n = 13; // Counter
    
    // Begin summations per Equation 12
    for (n=13; n<19; n++) {
        Hold      = C_star_n[n];
        Z_Factor += -D * Hold * pow(T,-u_n[n]);
        *dZdT    +=  D * u_n[n] * Hold * pow(T,-u_n[n]-1.0);
        *d2ZdT2  += -D * u_n[n] * (u_n[n]+1.0) * Hold * pow(T,-u_n[n]-2.0);
        *dZdd    -= Hold * pow(T, -u_n[n]);
    };
    
    for (n=13; n<59; n++) {
        Hold      = C_star_n[n] * (b_n[n]-c_n[n]*k_n[n]*pow(D,k_n[n])) * pow(D, b_n[n]) * exp(-c_n[n]*pow(D,k_n[n]));
        Z_Factor +=           pow(T,-u_n[n]) * Hold;
        *dZdT    += -u_n[n] * pow(T,-u_n[n]-1.0) * Hold;
        *d2ZdT2  +=  u_n[n] * (u_n[n]+1.0) * pow(T,-u_n[n]-2.0) * Hold;
        *dZdd    += C_star_n[n] * pow(T, -u_n[n]) * pow(D, b_n[n]) * exp(-c_n[n]*pow(D,k_n[n])) *
                    ( (-c_n[n]*pow(k_n[n],2.0)*pow(D,k_n[n]-1.0)) + (b_n[n]-c_n[n]*k_n[n]*pow(D,k_n[n]))*((b_n[n]/D)-(c_n[n]*k_n[n]*pow(D,k_n[n]-1.0))) );
    };
    
    *dZdd *= pow(K, 3.0);
    
    return Z_Factor;
}